from .param_parser_volume import *  # Module to analyze and break down input data into a structured, usable format
from .calculate_volume import *     # Module to calculate volume of different 3D shapes

def volume(*args):
    try:
        # Check if arguments are passed
        if not args:
            raise ValueError("No arguments were passed.")

        # Parse the input arguments
        dimensions, shape = param_parser(args)

        # Map shape names to their respective functions
        shape_functions = {
            "cube": cube,
            "cuboid": cuboid,
            "sphere": sphere,
            "cylinder": cylinder,
            "cone": cone,
            "pyramid": pyramid,
            "hemisphere": hemisphere,
            "prism": prism,
            "ellipsoid": ellipsoid,
        }

        # Get the appropriate function for the shape
        if shape not in shape_functions:
            raise ValueError(f"Unsupported shape: '{shape}'")

        # Call the shape-specific function
        result = shape_functions[shape](dimensions)
        return result

    except ValueError as ve:
        return f"ValueError: {ve}"

    except Exception as e:
        return f"Error: {str(e)}"



import math
from .unit_converter import * # Module to convert dimensions to base unit 'm'

def square(dictionary, target_unit = 'm'):
    
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for key, values in dictionary.items() for item in [key] + values]
    side_value = flat_list[1]
    side_unit = flat_list[0]
    
    # Convert side to target unit (default is meters)
    side = convert_to_base_unit(side_value, side_unit, target_unit)
    
    # Compute area in square of the target unit
    area = side * side
    result = f"Area: {area} {target_unit}²"
    # Returns the area of a square.
    return result

def rectangle(list, target_unit = 'm'):
    
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
    # '1' and '3' i.e. odd numbers have values and even numbers have keys (units)
    length_value = flat_list[1]
    length_unit = flat_list[0]
    breadth_value = flat_list[3]
    breadth_unit = flat_list[2]
    
    # Convert both to target unit (default is meters)
    length = convert_to_base_unit(length_value, length_unit, target_unit)
    breadth = convert_to_base_unit(breadth_value, breadth_unit, target_unit)
    
    # Compute area in rectangle of the target unit
    area = length * breadth
    result = f"Area: {area} {target_unit}²"
    # Returns the area of a rectangle
    return result

def circle(dictionary, target_unit = 'm'):
    
    flat_list = [item for key, values in dictionary.items() for item in [key] + values]
    radius_value = flat_list[1]
    radius_unit = flat_list[0]
    
    # Convert radius to target unit (default is meters)
    radius = convert_to_base_unit(radius_value, radius_unit, target_unit)
    
    # Compute area in square of the target unit
    area = math.pi * radius ** 2
    result = f"Area: {area} {target_unit}²"
    # Returns the area of a circle.
    return result

def triangle(list, target_unit = 'm'):
    
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
    # '1' and '3' i.e. odd numbers have values and even numbers have keys (units)
    l = len(flat_list)
    
    if l == 2:
        unit = flat_list[0]
        side = convert_to_base_unit(flat_list[1], unit, target_unit)
        area = (math.sqrt(3) / 4) * side ** 2
        result = f"Area: {area} {target_unit}²"
        return result
        
    elif l == 4:
        base_unit, base_value = flat_list[0], flat_list[1]
        height_unit, height_value = flat_list[2], flat_list[3]
        base = convert_to_base_unit(base_value, base_unit, target_unit)
        height = convert_to_base_unit(height_value, height_unit, target_unit)
        area = 0.5 * base * height
        result = f"Area: {area} {target_unit}²"
        return result
        
    elif l == 6:
        unit_1 = flat_list[0]
        unit_2 = flat_list[2]
        unit_3 = flat_list[4]
        value_1 = flat_list[1]
        value_2 = flat_list[3]
        value_3 = flat_list[5]
        
        if unit_1 == 'degrees' and value_1 < 180:
            angle_radians = math.radians(value_1)
            a = convert_to_base_unit(value_2, unit_2, target_unit)
            b = convert_to_base_unit(value_3, unit_3, target_unit)
            area = 0.5 * a * b * math.sin(angle_radians)
            result = f"Area: {area} {target_unit}²"
            return result
        
        elif unit_2 == 'degrees' and value_2 < 180:
            angle_radians = math.radians(value_2)
            a = convert_to_base_unit(value_1, unit_1, target_unit)
            b = convert_to_base_unit(value_3, unit_3, target_unit)
            area = 0.5 * a * b * math.sin(angle_radians)
            result = f"Area: {area} {target_unit}²"
            return result
        
        elif unit_3 == 'degrees' and value_3 < 180:
            angle_radians = math.radians(value_3)
            a = convert_to_base_unit(value_1, unit_1, target_unit)
            b = convert_to_base_unit(value_2, unit_2, target_unit)
            area = 0.5 * a * b * math.sin(angle_radians)
            result = f"Area: {area} {target_unit}²"
            return result
        
        else:
            a = convert_to_base_unit(value_1, unit_1, target_unit)
            b = convert_to_base_unit(value_2, unit_2, target_unit)
            c = convert_to_base_unit(value_3, unit_3, target_unit)
            s = (a + b + c)/2
            # Returns the area of a triangle when 3 sides are given
            area = math.sqrt(s * (s - a) * (s - b) * (s - c))
            result = f"Area: {area} {target_unit}²"
            return result
        
def parallelogram(list, target_unit = 'm'):
    
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
    # '1' and '3' i.e. odd numbers have values and even numbers have keys (units)
    l = len(flat_list)
    
    if l == 4:
        base_value = flat_list[1]
        base_unit = flat_list[0]
        height_value = flat_list[3]
        height_unit = flat_list[2]
        base = convert_to_base_unit(base_value, base_unit, target_unit)
        height = convert_to_base_unit(height_value, height_unit, target_unit)
        area = base * height
        result = f"Area: {area} {target_unit}²"
        return result
    
    elif l == 6:
        value_1 = flat_list[1]
        unit_1 = flat_list[0]
        value_2 = flat_list[3]
        unit_2 = flat_list[2]
        value_3 = flat_list[5]
        unit_3 = flat_list[4]
        
        if unit_1 == 'degrees' and value_1 < 180:
            angle_radians = math.radians(value_1)
            a = convert_to_base_unit(value_2, unit_2, target_unit)
            b = convert_to_base_unit(value_3, unit_3, target_unit)
            area = a * b * math.sin(angle_radians)
            result = f"Area: {area} {target_unit}²"
            return result
        
        elif unit_2 == 'degrees' and value_2 < 180:
            angle_radians = math.radians(value_2)
            a = convert_to_base_unit(value_1, unit_1, target_unit)
            b = convert_to_base_unit(value_3, unit_3, target_unit)
            area = a * b * math.sin(angle_radians)
            result = f"Area: {area} {target_unit}²"
            return result
        
        elif unit_3 == 'degrees' and value_3 < 180:
            angle_radians = math.radians(value_3)
            a = convert_to_base_unit(value_1, unit_1, target_unit)
            b = convert_to_base_unit(value_2, unit_2, target_unit)
            area = a * b * math.sin(angle_radians)
            result = f"Area: {area} {target_unit}²"
            return result

def rhombus(list, target_unit = 'm'):

    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
    # '1' and '3' i.e. odd numbers have values and even numbers have keys (units)
    l = len(flat_list)
    
    if l == 4:
        diagonal1_value = flat_list[1]
        diagonal1_unit = flat_list[0]
        diagonal2_value = flat_list[3]
        diagonal2_unit = flat_list[2]
        d1 = convert_to_base_unit(diagonal1_value, diagonal1_unit, target_unit)
        d2 = convert_to_base_unit(diagonal2_value, diagonal2_unit, target_unit)
        area = 0.5 * d1 * d2
        result = f"Area: {area} {target_unit}²"
        return result
    
    else:
        value_1 = flat_list[1]
        unit_1 = flat_list[0]
        value_2 = flat_list[3]
        unit_2 = flat_list[2]
        
        if unit_1 == 'degrees' and value_1 < 180:
            angle_radians = math.radians(value_1)
            a = convert_to_base_unit(value_2, unit_2, target_unit)
            area = a * a * math.sin(angle_radians)
            result = f"Area: {area} {target_unit}²"
            return result
        
        elif unit_2 == 'degrees' and value_2 < 180:
            angle_radians = math.radians(value_2)
            a = convert_to_base_unit(value_1, unit_1, target_unit)
            area = a * a * math.sin(angle_radians)
            result = f"Area: {area} {target_unit}²"
            return result
        
def trapezium(list, target_unit = 'm'):

    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
    # '1' and '3' i.e. odd numbers have values and even numbers have keys (units)
    l = len(flat_list)
    
    if l == 6:
        value_1 = flat_list[1]
        unit_1 = flat_list[0]
        value_2 = flat_list[3]
        unit_2 = flat_list[2]
        value_3 = flat_list[5]
        unit_3 = flat_list[4]
        a = convert_to_base_unit(value_1, unit_1, target_unit)
        b = convert_to_base_unit(value_2, unit_2, target_unit)
        h = convert_to_base_unit(value_3, unit_3, target_unit)
        area = 0.5 * (a + b) * h
        result = f"Area: {area} {target_unit}²"
        return result
    
    elif l== 8:
        value_1 = flat_list[1]
        unit_1 = flat_list[0]
        value_2 = flat_list[3]
        unit_2 = flat_list[2]
        value_3 = flat_list[5]
        unit_3 = flat_list[4]
        value_4 = flat_list[7]
        unit_4 = flat_list[6]
        
        if unit_1 == 'degrees' and value_1 < 180:
            angle_radians = math.radians(value_1)
            a = convert_to_base_unit(value_2, unit_2, target_unit)
            b = convert_to_base_unit(value_3, unit_3, target_unit)
            c = convert_to_base_unit(value_4, unit_4, target_unit)
            area = 0.5 * (a+b) * c * math.sin(angle_radians)
            result = f"Area: {area} {target_unit}²"
            return result
        
        elif unit_2 == 'degrees' and value_2 < 180:
            angle_radians = math.radians(value_2)
            a = convert_to_base_unit(value_1, unit_1, target_unit)
            b = convert_to_base_unit(value_3, unit_3, target_unit)
            c = convert_to_base_unit(value_4, unit_4, target_unit)
            area = 0.5 * (a+b) * c * math.sin(angle_radians)
            result = f"Area: {area} {target_unit}²"
            return result
        
        elif unit_3 == 'degrees' and value_3 < 180:
            angle_radians = math.radians(value_3)
            a = convert_to_base_unit(value_1, unit_1, target_unit)
            b = convert_to_base_unit(value_2, unit_2, target_unit)
            c = convert_to_base_unit(value_4, unit_4, target_unit)
            area = 0.5 * (a+b) * c * math.sin(angle_radians)
            result = f"Area: {area} {target_unit}²"
            return result
        
        elif unit_4 == 'degrees' and value_4 < 180:
            angle_radians = math.radians(value_4)
            a = convert_to_base_unit(value_1, unit_1, target_unit)
            b = convert_to_base_unit(value_2, unit_2, target_unit)
            c = convert_to_base_unit(value_3, unit_3, target_unit)
            area = 0.5 * (a+b) * c * math.sin(angle_radians)
            result = f"Area: {area} {target_unit}²"
            return result
        
    elif l== 10:
        value_1 = flat_list[1]
        unit_1 = flat_list[0]
        value_2 = flat_list[3]
        unit_2 = flat_list[2]
        value_3 = flat_list[5]
        unit_3 = flat_list[4]
        value_4 = flat_list[7]
        unit_4 = flat_list[6]
        value_5 = flat_list[9]
        unit_5 = flat_list[8]
        
        a = convert_to_base_unit(value_1, unit_1, target_unit)
        b = convert_to_base_unit(value_2, unit_2, target_unit)
        c = convert_to_base_unit(value_3, unit_3, target_unit)
        d = convert_to_base_unit(value_4, unit_4, target_unit)
        e = convert_to_base_unit(value_5, unit_5, target_unit)
        
        s1 = (a + b + e)/2
        A1 = math.sqrt(s1 * (s1 - a) * (s1 - b) * (s1 - e))
        s2 = (c + d + e)/2
        A2 = math.sqrt(s2 * (s2 - c) * (s2 - d) * (s2 - e))
        area = A1 + A2
        result = f"Area: {area} {target_unit}²"
        return result
    
def ellipse(list, target_unit = 'm'):

    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
    # '1' and '3' i.e. odd numbers have values and even numbers have keys (units)
    major_axis_value = flat_list[1]
    major_axis_unit = flat_list[0]
    minor_axis_value = flat_list[3]
    minor_axis_unit = flat_list[2]
    
    # Convert both to target unit (default is meters)
    major_axis = convert_to_base_unit(major_axis_value, major_axis_unit, target_unit)
    minor_axis = convert_to_base_unit(minor_axis_value, minor_axis_unit, target_unit)
    
    # Compute area in ellipse of the target unit
    area = math.pi * major_axis * minor_axis
    result = f"Area: {area} {target_unit}²"
    # Returns the area of a ellipse
    return result
        
                

    
    


def convert_to_base_unit(value, from_unit, base_unit='m'):
    conversion_factors = {
        'm': 1,
        'cm': 0.01,
        'mm': 0.001,
        'km': 1000,
        'inch': 0.0254,
        'foot': 0.3048,
        'yard': 0.9144
    }

    if from_unit not in conversion_factors or base_unit not in conversion_factors:
        raise ValueError(f"Unsupported unit: {from_unit} or {base_unit}")

    # Convert from 'from_unit' to meters
    value_in_meters = value * conversion_factors[from_unit]

    # Convert from meters to the desired 'base_unit'
    return value_in_meters / conversion_factors[base_unit]



# Validate 
def square_param_validator(args):
    if len(args) == 2:
        return True
    else:
        return False
    
def rectangle_param_validator(args):
    if len(args) == 3:
        return True
    else:
        return False
    
def circle_param_validator(args):
    if len(args) == 2:
        return True
    else:
        return False
    
def triangle_param_validator(args):
    if len(args) == 2:
        return True
    elif len(args) == 3:
        return True
    elif len(args) == 4:
        return True
    else:
        return False
    
def parallelogram_param_validator(args):
    if len(args) == 3:
        return True
    else:
        return False
    
def rhombus_param_validator(args):
    if len(args) == 2:
        return True
    else:
        return False
    
def trapezium_param_validator(args):
    if len(args) == 5:
        return True
    else:
        return False
    
def ellipse_param_validator(args):
    if len(args) == 3:
        return True
    else:
        return False
    
def cube_param_validator(args):
    if len(args) == 2:
        return True
    else:
        return False
    
def cuboid_param_validator(args):
    if len(args) == 4:
        return True
    else:
        return False
    
def sphere_param_validator(args):
    if len(args) == 2:
        return True
    else:
        return False
    
def cylinder_param_validator(args):
    if len(args) == 2:
        return True
    elif len(args) == 3:
        return True
    else:
        return False
    
def cone_param_validator(args):
    if len(args) == 2:
        return True
    elif len(args) == 3:
        return True
    else:
        return False
    
def pyramid_param_validator(args):
    if len(args) == 2:
        return True
    elif len(args) == 3:
        return True
    else:
        return False
    
def hemisphere_param_validator(args):
    if len(args) == 2:
        return True
    else:
        return False


import unittest
from mensurapy.surface_area import surface_area

class TestSurfaceAreaFunction(unittest.TestCase):
    def test_empty_parameter(self):
        self.assertEqual(surface_area(), "Error: No arguments were passed")
        
    def test_surface_area_cube(self):
        self.assertEqual(surface_area("cube", "3m"), " \n Total Surface Area: 54.0 m² \n Lateral Surface Area: 36.0 m² \n Base Area: 9.0 m²")
    
    def test_1_surface_area_cuboid(self):
        self.assertEqual(surface_area("cuboid", "4m", "3m", "2m"), " \n Total Surface Area: 52.0 m² \n Lateral Surface Area: 28.0 m² \n Base Area: 12.0 m²")
    
    def test_2_surface_area_cuboid(self):
        self.assertEqual(surface_area("cuboid", "4cm", "3m", "2cm"), " \n Total Surface Area: 0.3616 m² \n Lateral Surface Area: 0.1216 m² \n Base Area: 0.12 m²")
    
    def test_surface_area_sphere(self):
        self.assertEqual(surface_area("sphere", "3m"), " \n Total Surface Area: 113.09733552923255 m² \n Curved Surface Area: 113.09733552923255 m²")
    
    def test_1_surface_area_cylinder(self):
        self.assertEqual(surface_area("cylinder", "5m", "10m"), " \n Total Surface Area: 471.23889803846896 m² \n Lateral Surface Area: 314.1592653589793 m² \n Base Area: 78.53981633974483 m²")
        
    def test_2_surface_area_cylinder(self):
        self.assertEqual(surface_area("cylinder", "5cm", "10m"), " \n Total Surface Area: 3.1573006168577424 m² \n Lateral Surface Area: 3.141592653589793 m² \n Base Area: 0.007853981633974483 m²")
    
    def test_1_surface_area_cone(self):
        self.assertEqual(surface_area("cone", "4m", "3m"), " \n Total Surface Area: 113.09733552923255 m² \n Curved Surface Area: 62.83185307179586 m² \n Base Area: 50.26548245743669 m²")
    
    def test_2_surface_area_cone(self):
        self.assertEqual(surface_area("cone", "4m", "3cm"), " \n Total Surface Area: 100.53237861168765 m² \n Curved Surface Area: 50.26689615425097 m² \n Base Area: 50.26548245743669 m²")
    
    def test_1_surface_area_cuboid(self):
        self.assertEqual(surface_area("pyramid", "6m", "5m", "4m"), " \n Total Surface Area: 96.0 m² \n Lateral Surface Area: 60.0 m² \n Base Area: 36.0 m²")
    
    def test_surface_area_hemisphere(self):
        self.assertEqual(surface_area("sphere", "3m"), " \n Total Surface Area: 113.09733552923255 m² \n Curved Surface Area: 113.09733552923255 m²")    

if __name__ == '__main__':
    unittest.main()


from .param_parser_area import *  # Module to analyze and break down input data into a structured, usable format
from .calculate_area import * # Module to calculate area of different polygons

def area(*args):
    try:
        
        # First check if arguments are having values else display error
        if not args:  # Check if args is empty
            return("Error: No arguments were passed")
        
        # Parse arguments
        try:
            dimensions, shape = param_parser(args) # Call param_parser method to recieve parsed users inputs for calculation
        except Exception as e:
            return f"Error in parameter parsing: {e}"
        
        # Calculate area based on shape
        try:
            if shape == "square":
                result = square(dimensions)
                return result
            
            elif shape == "rectangle":
                result = rectangle(dimensions)
                return result
            
            elif shape == "circle":
                result = circle(dimensions)
                return result
            
            elif shape == "triangle":
                result = triangle(dimensions)
                return result
            
            elif shape == "parallelogram":
                result = parallelogram(dimensions)
                return result

            elif shape == "rhombus":
                result = rhombus(dimensions)
                return result
            
            elif shape == "trapezium":
                result = trapezium(dimensions)
                return result
            
            elif shape == "ellipse":
                result = ellipse(dimensions)
                return result
            
            else:
                return f"Error: Unsupported shape '{shape}'" 
            
        except Exception as e:
            return f"Error during area calculation for shape '{shape}': {e}"       
    
    except Exception as e:
        return f"Unexpected error: {e}"


import math
from .unit_converter import * # Module to convert dimensions to base unit 'm'

def cube(dictionary, target_unit = 'm'):
    
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for key, values in dictionary.items() for item in [key] + values]
    side_value = flat_list[1]
    side_unit = flat_list[0]
    
    # Convert side to target unit (default is meters)
    side = convert_to_base_unit(side_value, side_unit, target_unit)
    
    # Compute surface area in cube of the target unit
    TSA = 6 * side * side
    LSA = 4 * side * side
    BA = side * side
    result = f" \n Total Surface Area: {TSA} {target_unit}² \n Lateral Surface Area: {LSA} {target_unit}² \n Base Area: {BA} {target_unit}²"
    # Returns the surface area of a cube
    return result

def cuboid(list, target_unit = 'm'):
    
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
    # '1' and '3' i.e. odd numbers have values and even numbers have keys (units)
    length_value = flat_list[1]
    length_unit = flat_list[0]
    breadth_value = flat_list[3]
    breadth_unit = flat_list[2]
    height_value = flat_list[5]
    height_unit = flat_list[4]
    
    # Convert both to target unit (default is meters)
    l = convert_to_base_unit(length_value, length_unit, target_unit)
    b = convert_to_base_unit(breadth_value, breadth_unit, target_unit)
    h = convert_to_base_unit(height_value, height_unit, target_unit)
    
    # Compute surface area in cuboid of the target unit
    TSA = 2 * ((l * b) + (b * h) + (h * l))
    LSA = 2 * h * (l + b)
    BA = l * b
    result = f" \n Total Surface Area: {TSA} {target_unit}² \n Lateral Surface Area: {LSA} {target_unit}² \n Base Area: {BA} {target_unit}²"
    # Returns the surface area of a cuboid
    return result

def sphere(dictionary, target_unit = 'm'):
    
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for key, values in dictionary.items() for item in [key] + values]
    radius_value = flat_list[1]
    radius_unit = flat_list[0]
    
    # Convert side to target unit (default is meters)
    r = convert_to_base_unit(radius_value, radius_unit, target_unit)
    
    # Compute surface area in sphere of the target unit
    TSA = 4 * math.pi * r * r
    CSA = 4 * math.pi * r * r
    result = f" \n Total Surface Area: {TSA} {target_unit}² \n Curved Surface Area: {CSA} {target_unit}²"
    # Returns the surface area of a sphere
    return result

def cylinder(list, target_unit = 'm'):

    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
    # '1' and '3' i.e. odd numbers have values and even numbers have keys (units)
    radius_value = flat_list[1]
    radius_unit = flat_list[0]
    height_value = flat_list[3]
    height_unit = flat_list[2]
    
    # Convert both to target unit (default is meters)
    r = convert_to_base_unit(radius_value, radius_unit, target_unit)
    h = convert_to_base_unit(height_value, height_unit, target_unit)
    
    # Compute surface area in cylinder of the target unit
    TSA = 2 * math.pi * r * (r + h)
    CSA = 2 * math.pi * r * h
    BA = math.pi * r * r
    result = f" \n Total Surface Area: {TSA} {target_unit}² \n Lateral Surface Area: {CSA} {target_unit}² \n Base Area: {BA} {target_unit}²"
    # Returns the surface area of a cylinder
    return result

def cone(list, target_unit = 'm'):

    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
    # '1' and '3' i.e. odd numbers have values and even numbers have keys (units)
    radius_value = flat_list[1]
    radius_unit = flat_list[0]
    height_value = flat_list[3]
    height_unit = flat_list[2]
    
    # Convert both to target unit (default is meters)
    r = convert_to_base_unit(radius_value, radius_unit, target_unit)
    h = convert_to_base_unit(height_value, height_unit, target_unit)
    # slant height
    l = math.sqrt((r * r) + (h * h))
    
    # Compute surface area in cone of the target unit
    TSA = math.pi * r * (l + r) 
    CSA = math.pi * r * l
    BA = math.pi * r * r
    result = f" \n Total Surface Area: {TSA} {target_unit}² \n Curved Surface Area: {CSA} {target_unit}² \n Base Area: {BA} {target_unit}²"
    # Returns the surface area of a cone
    return result

def pyramid(list, target_unit = 'm'):

    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
    # '1' and '3' i.e. odd numbers have values and even numbers have keys (units)
    l = len(flat_list)
    
    if l == 2:
        unit = flat_list[0]
        side = convert_to_base_unit(flat_list[1], unit, target_unit)
        TSA = math.sqrt(3) * side ** 2
        BA = (math.sqrt(3) / 4) * side ** 2
        result = f" \n Total Surface Area: {TSA} {target_unit}² \n Base Area: {BA} {target_unit}²"
        return result
        
    elif l == 6:
        side_unit, side_value = flat_list[0], flat_list[1]
        slant_height_unit, slant_height_value = flat_list[2], flat_list[3]
        height_unit, height_value = flat_list[4], flat_list[5]
        s = convert_to_base_unit(side_value, side_unit, target_unit)
        l = convert_to_base_unit(slant_height_value, slant_height_unit, target_unit)
        h = convert_to_base_unit(height_value, height_unit, target_unit)
        TSA = (2 * s * l) + (s * s)
        LSA = 2 * s * l
        BA = s * s
        result = f" \n Total Surface Area: {TSA} {target_unit}² \n Lateral Surface Area: {LSA} {target_unit}² \n Base Area: {BA} {target_unit}²"
        return result
    
def hemisphere(dictionary, target_unit = 'm'):
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for key, values in dictionary.items() for item in [key] + values]
    radius_value = flat_list[1]
    radius_unit = flat_list[0]
    
    # Convert side to target unit (default is meters)
    r = convert_to_base_unit(radius_value, radius_unit, target_unit)
    
    # Compute surface area of hemisphere of the target unit
    TSA = 3 * math.pi * r * r
    CSA = 3 * math.pi * r * r
    result = f" \n Total Surface Area: {TSA} {target_unit}² \n Curved Surface Area: {CSA} {target_unit}²"
    # Returns the surface area of a hemisphere
    return result
    


from .param_parser_perimeter import *  # Module to analyze and break down input data into a structured, usable format
from .calculate_perimeter import * # Module to calculate perimeter of different polygons

def perimeter(*args):
    # First check if arguments are having values else display error
    if not args:  # Check if args is empty
        return "Error: No arguments were passed"
    else:
        dimensions, shape = param_parser(args) # Call param_parser method to recieve parsed users inputs for calculation
        
        if shape == "square":
            result = square(dimensions)
            return result
        
        if shape == "rectangle":
            result = rectangle(dimensions)
            return result
        
        if shape == "circle":
            result = circle(dimensions)
            return result
        
        if shape == "triangle":
            result = triangle(dimensions)
            return result
        
        if shape == "parallelogram":
            result = parallelogram(dimensions)
            return result
        
        if shape == "rhombus":
            result = rhombus(dimensions)
            return result
        
        if shape == "trapezium":
            result = trapezium(dimensions)
            return result
        
        if shape == "ellipse":
            result = ellipse(dimensions)
            return result
        
        if shape == "cube":
            result = cube(dimensions)
            return result
        
        if shape == "cuboid":
            result = cuboid(dimensions)
            return result
        
        if shape == "sphere":
            result = square(dimensions)
            
        if shape == "cylinder":
            result = cylinder(dimensions)
            return result
        
        if shape == "cone":
            result = cone(dimensions)
            return result
        
        if shape == "pyramid":
            result = pyramid(dimensions)
            return result
        
        if shape == "hemisphere":
            result = hemisphere(dimensions)
            return result


import math
from .unit_converter import *  # Module to convert dimensions to base unit 'm'

def cube(dictionary, target_unit='m'):
    try:
        flat_list = [item for key, values in dictionary.items() for item in [key] + values]
        side_value = flat_list[1]
        side_unit = flat_list[0]
        side = convert_to_base_unit(side_value, side_unit, target_unit)
        vol = side ** 3
        return f"Volume: {vol} {target_unit}³"
    except Exception as e:
        raise ValueError(f"Error calculating cube volume: {e}")

def cuboid(list, target_unit='m'):
    try:
        flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
        length_value = flat_list[1]
        length_unit = flat_list[0]
        breadth_value = flat_list[3]
        breadth_unit = flat_list[2]
        height_value = flat_list[5]
        height_unit = flat_list[4]
        l = convert_to_base_unit(length_value, length_unit, target_unit)
        b = convert_to_base_unit(breadth_value, breadth_unit, target_unit)
        h = convert_to_base_unit(height_value, height_unit, target_unit)
        vol = l * b * h
        return f"Volume: {vol} {target_unit}³"
    except Exception as e:
        raise ValueError(f"Error calculating cuboid volume: {e}")

def sphere(dictionary, target_unit='m'):
    try:
        flat_list = [item for key, values in dictionary.items() for item in [key] + values]
        radius_value = flat_list[1]
        radius_unit = flat_list[0]
        r = convert_to_base_unit(radius_value, radius_unit, target_unit)
        vol = (4/3) * math.pi * r ** 3
        return f"Volume: {vol} {target_unit}³"
    except Exception as e:
        raise ValueError(f"Error calculating sphere volume: {e}")

def cylinder(list, target_unit='m'):
    try:
        flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
        radius_value = flat_list[1]
        radius_unit = flat_list[0]
        height_value = flat_list[3]
        height_unit = flat_list[2]
        r = convert_to_base_unit(radius_value, radius_unit, target_unit)
        h = convert_to_base_unit(height_value, height_unit, target_unit)
        vol = math.pi * r ** 2 * h
        return f"Volume: {vol} {target_unit}³"
    except Exception as e:
        raise ValueError(f"Error calculating cylinder volume: {e}")

def cone(list, target_unit='m'):
    try:
        flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
        l = len(flat_list)
        if l == 4:
            radius_value = flat_list[1]
            radius_unit = flat_list[0]
            height_value = flat_list[3]
            height_unit = flat_list[2]
            r = convert_to_base_unit(radius_value, radius_unit, target_unit)
            h = convert_to_base_unit(height_value, height_unit, target_unit)
            vol = (1/3) * math.pi * r ** 2 * h
        elif l == 6:
            r1 = convert_to_base_unit(flat_list[1], flat_list[0], target_unit)
            r2 = convert_to_base_unit(flat_list[3], flat_list[2], target_unit)
            h = convert_to_base_unit(flat_list[5], flat_list[4], target_unit)
            vol = (1/3) * math.pi * h * (r1 ** 2 + r2 ** 2 + r1 * r2)
        else:
            raise ValueError("Invalid number of dimensions for cone")
        return f"Volume: {vol} {target_unit}³"
    except Exception as e:
        raise ValueError(f"Error calculating cone volume: {e}")

def pyramid(list, target_unit='m'):
    try:
        flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
        l = len(flat_list)
        if l == 4:
            b = convert_to_base_unit(flat_list[1], flat_list[0], target_unit)
            h = convert_to_base_unit(flat_list[3], flat_list[2], target_unit)
            vol = (1/3) * b ** 2 * h
        elif l == 6:
            l_val = convert_to_base_unit(flat_list[1], flat_list[0], target_unit)
            w = convert_to_base_unit(flat_list[3], flat_list[2], target_unit)
            h = convert_to_base_unit(flat_list[5], flat_list[4], target_unit)
            vol = (1/3) * l_val * w * h
        else:
            raise ValueError("Invalid number of dimensions for pyramid")
        return f"Volume: {vol} {target_unit}³"
    except Exception as e:
        raise ValueError(f"Error calculating pyramid volume: {e}")

def hemisphere(dictionary, target_unit='m'):
    try:
        flat_list = [item for key, values in dictionary.items() for item in [key] + values]
        r = convert_to_base_unit(flat_list[1], flat_list[0], target_unit)
        vol = (2/3) * math.pi * r ** 3
        return f"Volume: {vol} {target_unit}³"
    except Exception as e:
        raise ValueError(f"Error calculating hemisphere volume: {e}")

def prism(list, target_unit='m'):
    try:
        flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
        l = convert_to_base_unit(flat_list[1], flat_list[0], target_unit)
        b = convert_to_base_unit(flat_list[3], flat_list[2], target_unit)
        h = convert_to_base_unit(flat_list[5], flat_list[4], target_unit)
        vol = 0.5 * l * b * h
        return f"Volume: {vol} {target_unit}³"
    except Exception as e:
        raise ValueError(f"Error calculating prism volume: {e}")

def ellipsoid(list, target_unit='m'):
    try:
        flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
        a = convert_to_base_unit(flat_list[1], flat_list[0], target_unit)
        b = convert_to_base_unit(flat_list[3], flat_list[2], target_unit)
        c = convert_to_base_unit(flat_list[5], flat_list[4], target_unit)
        vol = (4/3) * math.pi * a * b * c
        return f"Volume: {vol} {target_unit}³"
    except Exception as e:
        raise ValueError(f"Error calculating ellipsoid volume: {e}")


import unittest
from mensurapy.area import area

class TestAreaFunction(unittest.TestCase):
    def test_empty_parameter(self):
        self.assertEqual(area(), "Error: No arguments were passed")
    
    def test_wrong_shape(self):
        self.assertEqual(area("trapezoid", "10m", "6m", "5m"), "Error in parameter parsing: too many values to unpack (expected 2)")        
    
    def test_area_square(self):
        self.assertEqual(area("square", "3m"), "Area: 9.0 m²")
        
    def test_area_rectangle1(self):
        self.assertEqual(area("rectangle", "3cm", "5cm"), "Area: 0.0015 m²")
        
    def test_area_rectangle2(self):
        self.assertEqual(area("rectangle", "3m", "5m"), "Area: 15.0 m²")
        
    def test_area_circle(self):
        self.assertEqual(area("circle", "3m"), "Area: 28.274333882308138 m²")
        
    def test_area_equilateral_triangle(self):
        self.assertEqual(area("triangle", "2m"), "Area: 1.7320508075688772 m²")
        
    def test_area_scalene_triangle(self):
        self.assertEqual(area("triangle", "2m", "3cm", "30degrees"), "Area: 0.014999999999999998 m²")
        
    def test_area_right_triangle(self):
        self.assertEqual(area("triangle", "3m", "400cm"), "Area: 6.0 m²")
        
    def test_area_triangle(self):
        self.assertEqual(area("triangle", "1.2m", "85cm", "0.00095km"), "Area: 0.4010922587136282 m²")
    
    def test_1_area_parallelogram(self):
        self.assertEqual(area("parallelogram", "10m", "6m"), "Area: 60.0 m²")
    
    def test_2_area_parallelogram(self):
        self.assertEqual(area("parallelogram", "8m", "5m", "60degrees"), "Area: 34.64101615137754 m²")
        
    def test_3_area_parallelogram(self):
        self.assertEqual(area("parallelogram", "8m", "5000mm", "60degrees"), "Area: 34.64101615137754 m²")

    def test_1_area_rhombus(self):
        self.assertEqual(area("rhombus", "8m", "6m"), "Area: 24.0 m²")
        
    def test_2_area_rhombus(self):
        self.assertEqual(area("rhombus", "800cm", "6m"), "Area: 24.0 m²")
        
    def test_1_area_trapezium(self):
        self.assertEqual(area("trapezium", "10m", "6m", "5m"), "Area: 40.0 m²")
        
    def test_2_area_trapezium(self):
        self.assertEqual(area("trapezium", "1000cm", "6m", "500cm"), "Area: 40.0 m²")
        
    def test_3_area_trapezium(self):
        self.assertEqual(area("trapezium", "8m", "4m", "6m", "60degrees"), "Area: 31.17691453623979 m²")
        
    def test_4_area_trapezium(self):
        self.assertEqual(area("trapezium", "8m", "400cm", "6m", "60degrees"), "Area: 31.17691453623979 m²")

    def test_5_area_trapezium(self):
        self.assertEqual(area("trapezium", "60degrees", "8m", "400cm", "6m"), "Area: 31.17691453623979 m²")

    def test_area_ellipse(self):
        self.assertEqual(area("ellipse", "5m", "3m"), "Area: 47.12388980384689 m²")

if __name__ == '__main__':
    unittest.main()



from .param_validator_perimeter import * # Module to validate arguments
import re # Module used to extract the user input values into values and units
def param_parser(args):
    
    # Recieving arguments
    # Based on the shape of polygon call the relevant method to split the arguments & list them
    try:
        match args[0]:
            case 'square':
                return square_param_parser(args), args[0]
            case 'rectangle':
                return rectangle_param_parser(args), args[0]
            case 'circle':
                return circle_param_parser(args), args[0]
            case 'triangle':
                return triangle_param_parser(args), args[0]
            case 'parallelogram':
                return parallelogram_param_parser(args), args[0]
            case 'rhombus':
                return rhombus_param_parser(args), args[0]
            case 'trapezium':
                return trapezium_param_parser(args), args[0]
            case 'ellipse':
                return ellipse_param_parser(args), args[0]
            case 'cube':
                return cube_param_parser(args), args[0] 
            case 'cuboid':
                return cuboid_param_parser(args), args[0]
            case 'sphere':
                return sphere_param_parser(args), args[0]
            case 'cylinder':
                return cylinder_param_parser(args), args[0]
            case 'cone':
                return cone_param_parser(args), args[0]
            case 'pyramid':
                return pyramid_param_parser(args), args[0]
            case 'hemisphere':
                return hemisphere_param_parser(args), args[0]   
            case _:
                return("Error: Invalid shape")
    except IndexError:
        print("Something went wrong- Index not in range!")
            
# Regex operation to determine the 'unit' and 'value' of the args recieved from method 'square_param_parser'
def extract_value_and_unit(value_string):
    # Regular expression to match the numeric value followed by a unit
    match = re.match(r"(\d+\.?\d*)\s*(\w+)", value_string)
    
    if match:
        value = float(match.group(1))  # Extract the value as a float
        unit = match.group(2)          # Extract the unit
        return value, unit
    else:
        return None, None  # If no match is found
    
# Packing the extracted parameters from method 'extract_value_and_unit'
def pack_value_and_unit(value, unit):
    # Initialise an empty dict
    data = {}

    # Adding multiple values under the same key manually
    data.setdefault(unit, []).append(value)
    return data    
            
def square_param_parser(args):
    
    """Call param_validator to check if the arguments are suitable for the calculation
    and if it returns True"""
    if square_param_validator(args):
        # extract argument 1 and send to extractor and recieve the 'value' and 'unit' splitted and made suitable for calculation
        side = args[1]
        value, unit = extract_value_and_unit(side)
    
        # Pack 'value' and 'unit' in 'data' to be used for calculation in area program
        data = pack_value_and_unit(value, unit)
        # Return extracted and packed 'value' and 'unit' to Module 'area'
        return data
    else:
        return {}
    
def rectangle_param_parser(args):
    # Initialising a list to append the extracted 'unit' and 'values' as there are more unit and value pairs
    data_packer = []
    """Call param_validator to check if the arguments are suitable for the calculation
    and if it returns True"""    
    if rectangle_param_validator(args):
        # extract arguments and send to extractor
        for i in range(len(args)-1):
            dimension = args[i+1]
            value, unit = extract_value_and_unit(dimension)
            data = pack_value_and_unit(value, unit)
            # Inserting the extracted value and unit in the initialised list
            data_packer.append(data)
        # Return extracted and packed 'value' and 'unit' to Module 'area'
        return data_packer
    else:
        return data_packer
    
def circle_param_parser(args):
    """Call param_validator to check if the arguments are suitable for the calculation
    and if it returns True"""
    if circle_param_validator(args):
        # extract argument 1 and send to extractor and recieve the 'value' and 'unit' splitted and made suitable for calculation
        print(args[1])
        radius = args[1]
        value, unit = extract_value_and_unit(radius)
    
        # Pack 'value' and 'unit' in 'data' to be used for calculation in area program
        data = pack_value_and_unit(value, unit)
        # Return extracted and packed 'value' and 'unit' to Module 'area'
        return data
    else:
        return {}
    
def triangle_param_parser(args):
    # Initialising a list to append the extracted 'unit' and 'values' as there are more unit and value pairs
    data_packer = []
    """Call param_validator to check if the arguments are suitable for the calculation
    and if it returns True"""    
    if triangle_param_validator(args):
        # extract arguments and send to extractor
        for i in range(len(args)-1):
            dimension = args[i+1]
            value, unit = extract_value_and_unit(dimension)
            data = pack_value_and_unit(value, unit)
            # Inserting the extracted value and unit in the initialised list
            data_packer.append(data)
        # Return extracted and packed 'value' and 'unit' to Module 'area'
        return data_packer
    else:
        return data_packer
    
def parallelogram_param_parser(args):
    # Initialising a list to append the extracted 'unit' and 'values' as there are more unit and value pairs
    data_packer = []
    """Call param_validator to check if the arguments are suitable for the calculation
    and if it returns True"""    
    if parallelogram_param_validator(args):
        # extract arguments and send to extractor
        for i in range(len(args)-1):
            dimension = args[i+1]
            value, unit = extract_value_and_unit(dimension)
            data = pack_value_and_unit(value, unit)
            # Inserting the extracted value and unit in the initialised list
            data_packer.append(data)
        # Return extracted and packed 'value' and 'unit' to Module 'area'
        return data_packer
    else:
        return data_packer
    
def rhombus_param_parser(args):
    
    """Call param_validator to check if the arguments are suitable for the calculation
    and if it returns True"""
    if rhombus_param_validator(args):
        # extract argument 1 and send to extractor and recieve the 'value' and 'unit' splitted and made suitable for calculation
        side = args[1]
        value, unit = extract_value_and_unit(side)
    
        # Pack 'value' and 'unit' in 'data' to be used for calculation in area program
        data = pack_value_and_unit(value, unit)
        # Return extracted and packed 'value' and 'unit' to Module 'area'
        return data
    else:
        return {}
    
def trapezium_param_parser(args):
    # Initialising a list to append the extracted 'unit' and 'values' as there are more unit and value pairs
    data_packer = []
    """Call param_validator to check if the arguments are suitable for the calculation
    and if it returns True"""    
    if trapezium_param_validator(args):
        # extract arguments and send to extractor
        for i in range(len(args)-1):
            dimension = args[i+1]
            value, unit = extract_value_and_unit(dimension)
            data = pack_value_and_unit(value, unit)
            # Inserting the extracted value and unit in the initialised list
            data_packer.append(data)
        # Return extracted and packed 'value' and 'unit' to Module 'area'
        return data_packer
    else:
        return data_packer
    
def ellipse_param_parser(args):
    # Initialising a list to append the extracted 'unit' and 'values' as there are more unit and value pairs
    data_packer = []
    """Call param_validator to check if the arguments are suitable for the calculation
    and if it returns True"""    
    if ellipse_param_validator(args):
        # extract arguments and send to extractor
        for i in range(len(args)-1):
            dimension = args[i+1]
            value, unit = extract_value_and_unit(dimension)
            data = pack_value_and_unit(value, unit)
            # Inserting the extracted value and unit in the initialised list
            data_packer.append(data)
        # Return extracted and packed 'value' and 'unit' to Module 'area'
        return data_packer
    else:
        return data_packer
    
def cube_param_parser(args):
    
    """Call param_validator to check if the arguments are suitable for the calculation
    and if it returns True"""
    if cube_param_validator(args):
        # extract argument 1 and send to extractor and recieve the 'value' and 'unit' splitted and made suitable for calculation
        side = args[1]
        value, unit = extract_value_and_unit(side)
    
        # Pack 'value' and 'unit' in 'data' to be used for calculation in area program
        data = pack_value_and_unit(value, unit)
        # Return extracted and packed 'value' and 'unit' to Module 'area'
        return data
    else:
        return {}
    
def cuboid_param_parser(args):
    # Initialising a list to append the extracted 'unit' and 'values' as there are more unit and value pairs
    data_packer = []
    """Call param_validator to check if the arguments are suitable for the calculation
    and if it returns True"""    
    if cuboid_param_validator(args):
        # extract arguments and send to extractor
        for i in range(len(args)-1):
            dimension = args[i+1]
            value, unit = extract_value_and_unit(dimension)
            data = pack_value_and_unit(value, unit)
            # Inserting the extracted value and unit in the initialised list
            data_packer.append(data)
        # Return extracted and packed 'value' and 'unit' to Module 'area'
        return data_packer
    else:
        return data_packer
    
def sphere_param_parser(args):
    
    """Call param_validator to check if the arguments are suitable for the calculation
    and if it returns True"""
    if sphere_param_validator(args):
        # extract argument 1 and send to extractor and recieve the 'value' and 'unit' splitted and made suitable for calculation
        side = args[1]
        value, unit = extract_value_and_unit(side)
    
        # Pack 'value' and 'unit' in 'data' to be used for calculation in area program
        data = pack_value_and_unit(value, unit)
        # Return extracted and packed 'value' and 'unit' to Module 'area'
        return data
    else:
        return {}
    
def cylinder_param_parser(args):
    # Initialising a list to append the extracted 'unit' and 'values' as there are more unit and value pairs
    data_packer = []
    """Call param_validator to check if the arguments are suitable for the calculation
    and if it returns True"""    
    if cylinder_param_validator(args):
        # extract arguments and send to extractor
        for i in range(len(args)-1):
            dimension = args[i+1]
            value, unit = extract_value_and_unit(dimension)
            data = pack_value_and_unit(value, unit)
            # Inserting the extracted value and unit in the initialised list
            data_packer.append(data)
        # Return extracted and packed 'value' and 'unit' to Module 'area'
        return data_packer
    else:
        return data_packer
    
def cone_param_parser(args):
    # Initialising a list to append the extracted 'unit' and 'values' as there are more unit and value pairs
    data_packer = []
    """Call param_validator to check if the arguments are suitable for the calculation
    and if it returns True"""    
    if cone_param_validator(args):
        # extract arguments and send to extractor
        for i in range(len(args)-1):
            dimension = args[i+1]
            value, unit = extract_value_and_unit(dimension)
            data = pack_value_and_unit(value, unit)
            # Inserting the extracted value and unit in the initialised list
            data_packer.append(data)
        # Return extracted and packed 'value' and 'unit' to Module 'area'
        return data_packer
    else:
        return data_packer
    
def pyramid_param_parser(args):
    # Initialising a list to append the extracted 'unit' and 'values' as there are more unit and value pairs
    data_packer = []
    """Call param_validator to check if the arguments are suitable for the calculation
    and if it returns True"""    
    if pyramid_param_validator(args):
        # extract arguments and send to extractor
        for i in range(len(args)-1):
            dimension = args[i+1]
            value, unit = extract_value_and_unit(dimension)
            data = pack_value_and_unit(value, unit)
            # Inserting the extracted value and unit in the initialised list
            data_packer.append(data)
        # Return extracted and packed 'value' and 'unit' to Module 'area'
        return data_packer
    else:
        return data_packer
    
def hemisphere_param_parser(args):
    
    """Call param_validator to check if the arguments are suitable for the calculation
    and if it returns True"""
    if hemisphere_param_validator(args):
        # extract argument 1 and send to extractor and recieve the 'value' and 'unit' splitted and made suitable for calculation
        side = args[1]
        value, unit = extract_value_and_unit(side)
    
        # Pack 'value' and 'unit' in 'data' to be used for calculation in area program
        data = pack_value_and_unit(value, unit)
        # Return extracted and packed 'value' and 'unit' to Module 'area'
        return data
    else:
        return {}
    



import unittest
from mensurapy.perimeter import perimeter

class TestPerimeterFunction(unittest.TestCase):
    def test_empty_parameter(self):
        self.assertEqual(perimeter(), "Error: No arguments were passed")
        
    def test_1_perimeter_square(self):
        self.assertEqual(perimeter("square", "3m"), "Perimeter: 12.0 m")
        
    def test_2_perimeter_square(self):
        self.assertEqual(perimeter("square", "3cm"), "Perimeter: 0.12 m")
        
    def test_1_perimeter_rectangle(self):
        self.assertEqual(perimeter("rectangle", "3m", "4m"), "Perimeter: 14.0 m")

    def test_2_perimeter_rectangle(self):
        self.assertEqual(perimeter("rectangle", "3cm", "4m"), "Perimeter: 8.06 m")
        
    def test_1_perimeter_circle(self):
        self.assertEqual(perimeter("circle", "3m"), "Perimeter: 18.84955592153876 m")    

    def test_2_perimeter_circle(self):
        self.assertEqual(perimeter("circle", "3mm"), "Perimeter: 0.01884955592153876 m")
        
    def test_1_perimeter_triangle(self):
        self.assertEqual(perimeter("triangle", "4m"), "Perimeter: 12.0 m")
        
    def test_2_perimeter_triangle(self):
        self.assertEqual(perimeter("triangle", "4m", "2cm", "4m"), "Perimeter: 8.02 m")
        
    def test_3_perimeter_triangle(self):
        self.assertEqual(perimeter("triangle", "4m", "2m"), "Perimeter: 10.0 m")
    
    def test_4_perimeter_triangle(self):
        self.assertEqual(perimeter("triangle", "4m", "2cm"), "Perimeter: 8.02 m")
        
    def test_perimeter_parallelogram(self):
        self.assertEqual(perimeter("parallelogram", "4cm", "2m"), "Perimeter: 4.08 m")
    
    def test_perimeter_rhombus(self):
        self.assertEqual(perimeter("rhombus", "4m"), "Perimeter: 16.0 m")
        
    def test_perimeter_trapezium(self):
        self.assertEqual(perimeter("trapezium", "4m", "2m", "4m", "3m"), "Perimeter: 13.0 m")
        
    def test_perimeter_ellipse(self):
        self.assertEqual(perimeter("triangle", "4m", "3m"), "Perimeter: 11.0 m")
        
    def test_1_perimeter_cube(self):
        self.assertEqual(perimeter("cube", "3m"), "Perimeter: 36.0 m")
        
    def test_2_perimeter_cube(self):
        self.assertEqual(perimeter("cube", "3cm"), "Perimeter: 0.36 m")
        
    def test_1_perimeter_cuboid(self):
        self.assertEqual(perimeter("cuboid", "5m", "3m", "2m"), "Perimeter: 40.0 m")
        
    def test_2_perimeter_cuboid(self):
        self.assertEqual(perimeter("cuboid", "5cm", "3cm", "2cm"), "Perimeter: 0.4 m")
        
    #def test_perimeter_sphere(self):
        #self.assertEqual(perimeter("sphere", "5m"), "AssertionError: None != 'Error: Perimeter not defined for sphere'")
        
    def test_1_perimeter_cylinder(self):
        self.assertEqual(perimeter("cylinder", "3m"), "Base Perimeter: 18.84955592153876 m")
        
    def test_2_perimeter_cylinder(self):
        self.assertEqual(perimeter("cylinder", "3m", "4m"), "Total linear boundary: 45.69911184307752 m")
        
    def test_3_perimeter_cylinder(self):
        self.assertEqual(perimeter("cylinder", "3cm", "10cm"), "Total linear boundary: 0.5769911184307752 m")
        
    def test_1_perimeter_cone(self):
        self.assertEqual(perimeter("cone", "4m"), "Base perimeter: 25.132741228718345 m")
        
    def test_2_perimeter_cone(self):
        self.assertEqual(perimeter("cone", "4m", "6m"), "Total boundary- 31.132741228718345 m")
        
    def test_perimeter_cone(self):
        self.assertEqual(perimeter("pyramid", "6m", "5m"), "Total boundary- 44.0 m")
        
    def test_perimeter_hemisphere(self):
        self.assertEqual(perimeter("hemisphere", "5m"), "Perimeter: 31.41592653589793 m")
        
if __name__ == '__main__':
    unittest.main()


from .param_validator_volume import *  # Module to validate arguments
import re  # Module used to extract the user input values into values and units

def param_parser(args):
    try:
        shape = args[0]
        match shape:
            case 'cube':
                return cube_param_parser(args), shape
            case 'cuboid':
                return cuboid_param_parser(args), shape
            case 'sphere':
                return sphere_param_parser(args), shape
            case 'cylinder':
                return cylinder_param_parser(args), shape
            case 'cone':
                return cone_param_parser(args), shape
            case 'pyramid':
                return pyramid_param_parser(args), shape
            case 'hemisphere':
                return hemisphere_param_parser(args), shape
            case 'prism':
                return prism_param_parser(args), shape
            case 'ellipsoid':
                return ellipsoid_param_parser(args), shape
            case _:
                raise ValueError(f"Invalid shape '{shape}' provided.")
    except IndexError:
        raise IndexError("Shape not provided. Please pass at least one argument.")
    except Exception as e:
        raise Exception(f"Error in param_parser: {e}")

def extract_value_and_unit(value_string):
    try:
        match = re.match(r"(\d+\.?\d*)\s*(\w+)", value_string)
        if match:
            value = float(match.group(1))
            unit = match.group(2)
            return value, unit
        else:
            raise ValueError(f"Input '{value_string}' is not in the expected format like '5m' or '3 kg'")
    except Exception as e:
        raise Exception(f"Error in extract_value_and_unit: {e}")

def pack_value_and_unit(value, unit):
    try:
        data = {}
        data.setdefault(unit, []).append(value)
        return data
    except Exception as e:
        raise Exception(f"Error in pack_value_and_unit: {e}")

def cube_param_parser(args):
    try:
        if cube_param_validator(args):
            side = args[1]
            value, unit = extract_value_and_unit(side)
            data = pack_value_and_unit(value, unit)
            return data
        else:
            raise ValueError("Invalid arguments for cube.")
    except IndexError:
        raise IndexError("Missing dimension for cube.")
    except Exception as e:
        raise Exception(f"cube_param_parser error: {e}")

def cuboid_param_parser(args):
    data_packer = []
    try:
        if cuboid_param_validator(args):
            for dimension in args[1:]:
                value, unit = extract_value_and_unit(dimension)
                data = pack_value_and_unit(value, unit)
                data_packer.append(data)
            return data_packer
        else:
            raise ValueError("Invalid arguments for cuboid.")
    except Exception as e:
        raise Exception(f"cuboid_param_parser error: {e}")

def sphere_param_parser(args):
    try:
        if sphere_param_validator(args):
            side = args[1]
            value, unit = extract_value_and_unit(side)
            data = pack_value_and_unit(value, unit)
            return data
        else:
            raise ValueError("Invalid arguments for sphere.")
    except Exception as e:
        raise Exception(f"sphere_param_parser error: {e}")

def cylinder_param_parser(args):
    data_packer = []
    try:
        if cylinder_param_validator(args):
            for dimension in args[1:]:
                value, unit = extract_value_and_unit(dimension)
                data = pack_value_and_unit(value, unit)
                data_packer.append(data)
            return data_packer
        else:
            raise ValueError("Invalid arguments for cylinder.")
    except Exception as e:
        raise Exception(f"cylinder_param_parser error: {e}")

def cone_param_parser(args):
    data_packer = []
    try:
        if cone_param_validator(args):
            for dimension in args[1:]:
                value, unit = extract_value_and_unit(dimension)
                data = pack_value_and_unit(value, unit)
                data_packer.append(data)
            return data_packer
        else:
            raise ValueError("Invalid arguments for cone.")
    except Exception as e:
        raise Exception(f"cone_param_parser error: {e}")

def pyramid_param_parser(args):
    data_packer = []
    try:
        if pyramid_param_validator(args):
            for dimension in args[1:]:
                value, unit = extract_value_and_unit(dimension)
                data = pack_value_and_unit(value, unit)
                data_packer.append(data)
            return data_packer
        else:
            raise ValueError("Invalid arguments for pyramid.")
    except Exception as e:
        raise Exception(f"pyramid_param_parser error: {e}")

def hemisphere_param_parser(args):
    try:
        if hemisphere_param_validator(args):
            side = args[1]
            value, unit = extract_value_and_unit(side)
            data = pack_value_and_unit(value, unit)
            return data
        else:
            raise ValueError("Invalid arguments for hemisphere.")
    except Exception as e:
        raise Exception(f"hemisphere_param_parser error: {e}")

def prism_param_parser(args):
    data_packer = []
    try:
        if prism_param_validator(args):
            for dimension in args[1:]:
                value, unit = extract_value_and_unit(dimension)
                data = pack_value_and_unit(value, unit)
                data_packer.append(data)
            return data_packer
        else:
            raise ValueError("Invalid arguments for prism.")
    except Exception as e:
        raise Exception(f"prism_param_parser error: {e}")

def ellipsoid_param_parser(args):
    data_packer = []
    try:
        if ellipsoid_param_validator(args):
            for dimension in args[1:]:
                value, unit = extract_value_and_unit(dimension)
                data = pack_value_and_unit(value, unit)
                data_packer.append(data)
            return data_packer
        else:
            raise ValueError("Invalid arguments for ellipsoid.")
    except Exception as e:
        raise Exception(f"ellipsoid_param_parser error: {e}")



# Validate 
def cube_param_validator(args):
    if len(args) == 2:
        return True
    else:
        return False
    
def cuboid_param_validator(args):
    if len(args) == 4:
        return True 
    else:
        return False
    
def sphere_param_validator(args):
    if len(args) == 2:
        return True
    else:
        return False
    
def cylinder_param_validator(args):
    if len(args) == 3:
        return True
    else:
        return False
    
def cone_param_validator(args):
    if len(args) == 3:
        return True
    else:
        return False
    
def pyramid_param_validator(args):
    if len(args) == 4:
        return True  
    elif len(args) == 2:
        return True 
    else:
        return False
    
def hemisphere_param_validator(args):
    if len(args) == 2:
        return True
    else:
        return False


from .param_parser_surface_area import *  # Module to analyze and break down input data into a structured, usable format
from .calculate_surface_area import * # Module to calculate surface area of different polygons

def surface_area(*args):
    # First check if arguments are having values else display error
    if not args:  # Check if args is empty
        return "Error: No arguments were passed"
    else:
        dimensions, shape = param_parser(args) # Call param_parser method to recieve parsed users inputs for calculation
        
        if shape == "cube":
            result = cube(dimensions)
            return result
        
        if shape == "cuboid":
            result = cuboid(dimensions)
            return result
        
        if shape == "sphere":
            result = sphere(dimensions)
            return result
        
        if shape == "cylinder":
            result = cylinder(dimensions)
            return result
        
        if shape == "cone":
            result = cone(dimensions)
            return result
        
        if shape == "pyramid":
            result = pyramid(dimensions)
            return result
        
        if shape == "hemisphere":
            result = hemisphere(dimensions)
            return result


import math
from .unit_converter import * # Module to convert dimensions to base unit 'm'

def square(dictionary, target_unit = 'm'):
    
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for key, values in dictionary.items() for item in [key] + values]
    side_value = flat_list[1]
    side_unit = flat_list[0]
    
    # Convert side to target unit (default is meters)
    side = convert_to_base_unit(side_value, side_unit, target_unit)
    
    # Compute perimeter of a square in the target unit
    perimeter = 4 * side
    result = f"Perimeter: {perimeter} {target_unit}"
    # Returns the perimeter of a square.
    return result

def rectangle(list, target_unit = 'm'):
    
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
    # '1' and '3' i.e. odd numbers have values and even numbers have keys (units)
    length_value = flat_list[1]
    length_unit = flat_list[0]
    breadth_value = flat_list[3]
    breadth_unit = flat_list[2]
    
    # Convert both to target unit (default is meters)
    length = convert_to_base_unit(length_value, length_unit, target_unit)
    breadth = convert_to_base_unit(breadth_value, breadth_unit, target_unit)
    
    # Compute perimeter in rectangle of the target unit
    perimeter = 2 * (length + breadth)
    result = f"Perimeter: {perimeter} {target_unit}"
    # Returns the perimeter of a rectangle
    return result

def circle(dictionary, target_unit = 'm'):
    
    flat_list = [item for key, values in dictionary.items() for item in [key] + values]
    radius_value = flat_list[1]
    radius_unit = flat_list[0]
    
    # Convert radius to target unit (default is meters)
    radius = convert_to_base_unit(radius_value, radius_unit, target_unit)
    
    # Compute perimeter in square of the target unit
    perimeter = 2 * math.pi * radius
    result = f"Perimeter: {perimeter} {target_unit}"
    # Returns the perimeter of a circle.
    return result

def triangle(list, target_unit = 'm'):
    
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
    # '1' and '3' i.e. odd numbers have values and even numbers have keys (units)
    l = len(flat_list)
    
    if l == 2:
        unit = flat_list[0]
        side = convert_to_base_unit(flat_list[1], unit, target_unit)
        perimeter = 3 * side
        result = f"Perimeter: {perimeter} {target_unit}"
        return result
    
    elif l == 4:
        unit_1 = flat_list[0]
        side_1 = convert_to_base_unit(flat_list[1], unit_1, target_unit)
        unit_2 = flat_list[2]
        side_2 = convert_to_base_unit(flat_list[3], unit_2, target_unit)
        perimeter = (2 * side_1) + side_2
        result = f"Perimeter: {perimeter} {target_unit}"
        return result
    
    elif l == 6:
        unit_1 = flat_list[0]
        side_1 = convert_to_base_unit(flat_list[1], unit_1, target_unit)
        unit_2 = flat_list[2]
        side_2 = convert_to_base_unit(flat_list[3], unit_2, target_unit)
        unit_3 = flat_list[4]
        side_3 = convert_to_base_unit(flat_list[5], unit_3, target_unit)
        perimeter = side_1 + side_2 + side_3
        result = f"Perimeter: {perimeter} {target_unit}"
        return result
    
def parallelogram(list, target_unit = 'm'):
    
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
    # '1' and '3' i.e. odd numbers have values and even numbers have keys (units)
    length_value = flat_list[1]
    length_unit = flat_list[0]
    breadth_value = flat_list[3]
    breadth_unit = flat_list[2]
    
    # Convert both to target unit (default is meters)
    a = convert_to_base_unit(length_value, length_unit, target_unit)
    b = convert_to_base_unit(breadth_value, breadth_unit, target_unit)
    
    # Compute perimeter in parallelogram of the target unit
    perimeter = 2 * (a + b)
    result = f"Perimeter: {perimeter} {target_unit}"
    # Returns the perimeter of a parallelogram
    return result

def rhombus(dictionary, target_unit = 'm'):
    
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for key, values in dictionary.items() for item in [key] + values]
    side_value = flat_list[1]
    side_unit = flat_list[0]
    
    # Convert side to target unit (default is meters)
    side = convert_to_base_unit(side_value, side_unit, target_unit)
    
    # Compute perimeter of a rhombus in the target unit
    perimeter = 4 * side
    result = f"Perimeter: {perimeter} {target_unit}"
    # Returns the perimeter of a rhombus.
    return result

def trapezium(list, target_unit = 'm'):
    
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
    # '1' and '3' i.e. odd numbers have values and even numbers have keys (units)
    value_1 = flat_list[1]
    unit_1 = flat_list[0]
    value_2 = flat_list[3]
    unit_2 = flat_list[2]
    value_3 = flat_list[5]
    unit_3 = flat_list[4]
    value_4 = flat_list[7]
    unit_4 = flat_list[6]
    
    # Convert both to target unit (default is meters)
    a = convert_to_base_unit(value_1, unit_1, target_unit)
    b = convert_to_base_unit(value_2, unit_2, target_unit)
    c = convert_to_base_unit(value_3, unit_3, target_unit)
    d = convert_to_base_unit(value_4, unit_4, target_unit)
    
    # Compute perimeter in trapezium of the target unit
    perimeter = a + b + c + d
    result = f"Perimeter: {perimeter} {target_unit}"
    # Returns the perimeter of a trapezium
    return result

def ellipse(list, target_unit = 'm'):
    
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
    # '1' and '3' i.e. odd numbers have values and even numbers have keys (units)
    semi_major_axis_value = flat_list[1]
    semi_major_axis_unit = flat_list[0]
    semi_minor_axis_value = flat_list[3]
    semi_minor_axis_unit = flat_list[2]
    
    # Convert both to target unit (default is meters)
    a = convert_to_base_unit(semi_major_axis_value, semi_major_axis_unit, target_unit)
    b = convert_to_base_unit(semi_minor_axis_value, semi_minor_axis_unit, target_unit)
    
    # Compute perimeter in rectangle of the target unit
    perimeter = math.pi * (3 * (a + b) - math.sqrt((3 * a + b) * (a + 3 * b)))
    result = f"Perimeter: {perimeter} {target_unit}"
    # Returns the perimeter of a rectangle
    return result

def cube(dictionary, target_unit = 'm'):
    
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for key, values in dictionary.items() for item in [key] + values]
    side_value = flat_list[1]
    side_unit = flat_list[0]
    
    # Convert side to target unit (default is meters)
    side = convert_to_base_unit(side_value, side_unit, target_unit)
    
    # Compute perimeter of a cube in the target unit
    perimeter = 12 * side
    result = f"Perimeter: {perimeter} {target_unit}"
    # Returns the perimeter of a cube.
    return result

def cuboid(list, target_unit = 'm'):
    
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
    # '1' and '3' i.e. odd numbers have values and even numbers have keys (units)
    length_value = flat_list[1]
    length_unit = flat_list[0]
    breadth_value = flat_list[3]
    breadth_unit = flat_list[2]
    height_value = flat_list[5]
    height_unit = flat_list[4]
    
    # Convert both to target unit (default is meters)
    l = convert_to_base_unit(length_value, length_unit, target_unit)
    b = convert_to_base_unit(breadth_value, breadth_unit, target_unit)
    h = convert_to_base_unit(height_value, height_unit, target_unit)
    
    # Compute perimeter in cuboid of the target unit
    perimeter = 4 * (l + b + h)
    result = f"Perimeter: {perimeter} {target_unit}"
    # Returns the perimeter of a cuboid
    return result

def sphere(dictionary, target_unit = 'm'):
    
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for key, values in dictionary.items() for item in [key] + values]
    radius = flat_list[1]
    radius_unit = flat_list[0]
    result = " "
    # Returns the empty string
    return result

def cylinder(list, target_unit = 'm'):
    
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
    # '1' and '3' i.e. odd numbers have values and even numbers have keys (units)
    l = len(flat_list)
    
    if l == 2:
        unit = flat_list[0]
        r = convert_to_base_unit(flat_list[1], unit, target_unit)
        perimeter = 2 * math.pi * r
        result = f"Base Perimeter: {perimeter} {target_unit}"
        return result
    
    elif l == 4:
        r_unit = flat_list[0]
        r = convert_to_base_unit(flat_list[1], r_unit, target_unit)
        h_unit = flat_list[2]
        h = convert_to_base_unit(flat_list[3], h_unit, target_unit)
        perimeter = 2 * ((2 * math.pi * r) + h)
        result = f"Total linear boundary: {perimeter} {target_unit}"
        return result
    
def cone(list, target_unit = 'm'):
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
    # '1' and '3' i.e. odd numbers have values and even numbers have keys (units)
    l = len(flat_list)
    
    if l == 2:
        unit = flat_list[0]
        r = convert_to_base_unit(flat_list[1], unit, target_unit)
        perimeter = 2 * math.pi * r
        result = f"Base perimeter: {perimeter} {target_unit}"
        return result
    
    elif l == 4:
        r_unit = flat_list[0]
        r = convert_to_base_unit(flat_list[1], r_unit, target_unit)
        l_unit = flat_list[2]
        l = convert_to_base_unit(flat_list[3], l_unit, target_unit)
        perimeter = 2 * math.pi * r + l
        result = f"Total boundary- {perimeter} {target_unit}"
        return result
    
def pyramid(list, target_unit = 'm'):
    
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for d in list for key, values in d.items() for item in [key] + values]
    # '1' and '3' i.e. odd numbers have values and even numbers have keys (units)
    l = len(flat_list)
    
    if l == 2:
        unit = flat_list[0]
        side = convert_to_base_unit(flat_list[1], unit, target_unit)
        perimeter = 2 * math.pi * side
        result = f"Base perimeter: {perimeter} {target_unit}"
        return result
    
    elif l == 4:
        side_unit = flat_list[0]
        side = convert_to_base_unit(flat_list[1], side_unit, target_unit)
        l_unit = flat_list[2]
        l = convert_to_base_unit(flat_list[3], l_unit, target_unit)
        perimeter = 4 * side + 4 * l
        result = f"Total boundary- {perimeter} {target_unit}"
        return result

def hemisphere(dictionary, target_unit = 'm'):
    # Structure the recieved data in a flattened list displaying keys and values
    flat_list = [item for key, values in dictionary.items() for item in [key] + values]
    radius_value = flat_list[1]
    radius_unit = flat_list[0]
    
    # Convert side to target unit (default is meters)
    r = convert_to_base_unit(radius_value, radius_unit, target_unit)
    
    # Compute perimeter of a hemisphere in the target unit
    perimeter = 2 * math.pi * r
    result = f"Perimeter: {perimeter} {target_unit}"
    # Returns the perimeter of a hemisphere.
    return result
    









from setuptools import setup, find_packages

setup(
    name="mensurapy",
    version="0.1",
    description="This package provides a comprehensive set of functions to calculate the area and perimeter of 2D shapes, as well as the surface area and volume of 3D geometric shapes. It supports various units of measurement and is designed to simplify geometric computations for students, educators, and developers alike.",
    long_description="",
    long_description_content_type='text/markdown',
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.12",
        "Operating System :: OS Independent",
    ],
    install_requires=[],  # List dependencies here
    author="Anuva Das",
    author_email="anuvadas@hotmail.com",
    url="https://github.com/Anuva13/MensuraPy.git",
    license="MIT",
)


import unittest
from mensurapy.volume import volume

class TestVolumeFunction(unittest.TestCase):

    def test_volume_cube(self):
        self.assertEqual(volume("cube", "3m"), "Volume: 27.0 m³")
        self.assertEqual(volume("cube", "300cm"), "Volume: 27.0 m³")
        self.assertEqual(volume("cube", "3000mm"), "Volume: 27.0 m³")

    def test_volume_cuboid(self):
        self.assertEqual(volume("cuboid", "2m", "3m", "4m"), "Volume: 24.0 m³")
        self.assertEqual(volume("cuboid", "200cm", "300cm", "400cm"), "Volume: 24.0 m³")
        self.assertEqual(volume("cuboid", "2000mm", "3000mm", "4000mm"), "Volume: 24.0 m³")

    def test_volume_sphere(self):
        self.assertEqual(volume("sphere", "3m"), "Volume: 113.09733552923254 m³")
        self.assertEqual(volume("sphere", "300cm"), "Volume: 113.09733552923254 m³")
        self.assertEqual(volume("sphere", "3000mm"), "Volume: 113.09733552923254 m³")

    def test_volume_cylinder(self):
        self.assertEqual(volume("cylinder", "2m", "3m"), "Volume: 37.69911184307752 m³")
        self.assertEqual(volume("cylinder", "200cm", "300cm"), "Volume: 37.69911184307752 m³")
        self.assertEqual(volume("cylinder", "2000mm", "3000mm"), "Volume: 37.69911184307752 m³")

    def test_volume_cone(self):
        self.assertEqual(volume("cone", "2m", "3m"), "Volume: 12.566370614359172 m³")
        self.assertEqual(volume("cone", "200cm", "300cm"), "Volume: 12.566370614359172 m³")
        self.assertEqual(volume("cone", "2000mm", "3000mm"), "Volume: 12.566370614359172 m³")

    def test_volume_hemisphere(self):
        self.assertEqual(volume("hemisphere", "3m"), "Volume: 56.54866776461627 m³")
        self.assertEqual(volume("hemisphere", "300cm"), "Volume: 56.54866776461627 m³")
        self.assertEqual(volume("hemisphere", "3000mm"), "Volume: 56.54866776461627 m³")

    def test_volume_pyramid(self):
        self.assertEqual(volume("pyramid", "3m", "4m", "5m"), "Volume: 20.0 m³")
        self.assertEqual(volume("pyramid", "300cm", "400cm", "500cm"), "Volume: 20.0 m³")
        self.assertEqual(volume("pyramid", "3000mm", "4000mm", "5000mm"), "Volume: 20.0 m³")

    def test_volume_prism(self):
        self.assertEqual(volume("prism", "3m", "4m", "10m" ), "Volume: 60.0 m³")
        self.assertEqual(volume("prism", "300cm", "400cm", "1000cm"), "Volume: 60.0 m³")
        self.assertEqual(volume("prism", "3000mm", "4000mm", "10000mm"), "Volume: 60.0 m³")

    def test_volume_ellipsoid(self):
        self.assertEqual(volume("ellipsoid", "2m", "3m", "4m"), "Volume: 100.53096491487338 m³")
        self.assertEqual(volume("ellipsoid", "200cm", "300cm", "400cm"), "Volume: 100.53096491487338 m³")
        self.assertEqual(volume("ellipsoid", "2000mm", "3000mm", "4000mm"), "Volume: 100.53096491487338 m³")

    def test_invalid_shape(self):
        self.assertEqual(volume("invalidshape", "3m"), "Error: Error in param_parser: Invalid shape 'invalidshape' provided.")

    def test_no_arguments(self):
        self.assertEqual(volume(), "ValueError: No arguments were passed.")

if __name__ == "__main__":
    unittest.main()
